"""Defines map of RPC responses by service and method name."""

from ansys.api.edb.v1.bondwire_def_pb2 import Jedec5BondwireDefParametersMessage
from ansys.api.edb.v1.bondwire_pb2 import (
    BondwireTrajMessage,
    BondwireTypeMessage,
    GetCrossSectionTypeMessage,
)
from ansys.api.edb.v1.circle_pb2 import CircleParametersMessage
from ansys.api.edb.v1.component_model_pb2 import ComponentModelTypeMessage
from ansys.api.edb.v1.component_property_pb2 import ComponentPropModelMessage
from ansys.api.edb.v1.connectable_pb2 import LayoutObjTypeMessage
from ansys.api.edb.v1.database_pb2 import GetVersionMessage
from ansys.api.edb.v1.debye_model_pb2 import FrequencyRangeMessage
from ansys.api.edb.v1.die_property_pb2 import DieOrientationMessage, DieTypeMessage
from ansys.api.edb.v1.differential_pair_pb2 import DifferentialPairNetRefsMessage
from ansys.api.edb.v1.edb_error_manager_pb2 import EDBErrorMessages
from ansys.api.edb.v1.edb_messages_pb2 import (
    ComplexMessage,
    ComponentTypeMessage,
    DesignModeMessage,
    DoublesMessage,
    EDBInternalIdMessage,
    EDBObjCollectionMessage,
    EDBObjMessage,
    EDBObjPairMessage,
    HfssExtentInfoMessage,
    IntegersPropertyMessage,
    ProductPropertyIdsMessage,
    StringMapMessage,
    StringPairsMessage,
    StringsMessage,
    TemperatureSettingsMessage,
    TouchstoneExportSettingsMessage,
    ValueMessage,
    ValuePairMessage,
)
from ansys.api.edb.v1.edge_term_pb2 import EdgeParamsMessage, EdgeTypeMessage
from ansys.api.edb.v1.group_pb2 import GroupTypeMessage
from ansys.api.edb.v1.hfss_pi_simulation_settings_pb2 import HFSSPIModelTypeMessage
from ansys.api.edb.v1.hfss_simulation_settings_pb2 import (
    AdaptTypeMessage,
    BasisFunctionOrderMessage,
    BroadbandFrequencyAdaptiveSolutionMessage,
    MultiFrequencyAdaptiveSolutionMessage,
    SingleFrequencyAdaptiveSolutionMessage,
    SolverTypeMessage,
)
from ansys.api.edb.v1.hfss_simulation_setup_pb2 import MeshOperationsMessage
from ansys.api.edb.v1.io_manager_pb2 import ResolvedFuturesMessage
from ansys.api.edb.v1.layer_collection_pb2 import (
    LayerCollectionModeMessage,
    TopBottomStackupLayersMessage,
)
from ansys.api.edb.v1.layer_pb2 import (
    DrawOverrideMessage,
    LayerTypeMessage,
    TopBottomAssociationMessage,
    ZonesMessage,
)
from ansys.api.edb.v1.layout_instance_pb2 import LayoutObjInstancesQueryResultsMessage
from ansys.api.edb.v1.layout_obj_instance_3d_geometry_pb2 import TesselationDataMessage
from ansys.api.edb.v1.layout_obj_instance_pb2 import FetchedLayoutObjInstanceGeometriesMessage
from ansys.api.edb.v1.layout_pb2 import LayoutPortRefTerminalsConnectedMessage
from ansys.api.edb.v1.material_def_pb2 import (
    MaterialDefGetAllPropertiesMessage,
    MaterialDefGetDimensionMessage,
)
from ansys.api.edb.v1.material_property_thermal_modifier_pb2 import (
    MaterialPropertyThermalModifierParamsMessage,
)
from ansys.api.edb.v1.mcad_model_pb2 import McadModelRotationMessage
from ansys.api.edb.v1.multipole_debye_model_pb2 import MultipoleDebyeModelGetParams
from ansys.api.edb.v1.package_def_pb2 import HeatSinkMessage
from ansys.api.edb.v1.padstack_def_data_pb2 import (
    PadstackDefDataConnectionPtMessage,
    PadstackDefDataGetLayerIdsMessage,
    PadstackDefDataGetLayerNamesMessage,
    PadstackDefDataGetSolderBallParamMessage,
    PadstackDefDataPadParametersMessage,
    PadstackDefDataPadstackHoleRangeMessage,
    PadstackDefDataSolderballPlacementMessage,
    PadstackDefDataSolderballShapeMessage,
)
from ansys.api.edb.v1.padstack_inst_term_pb2 import PadstackInstTermParamsMessage
from ansys.api.edb.v1.padstack_instance_pb2 import (
    PadstackInstBackDrillByDepthMessage,
    PadstackInstBackDrillByLayerMessage,
    PadstackInstBackDrillTypeMessage,
    PadstackInstHoleOverridesMessage,
    PadstackInstLayerRangeMessage,
    PadstackInstPositionAndRotationMessage,
)
from ansys.api.edb.v1.path_pb2 import (
    ClipInfoMessage,
    CornerStyleMessage,
    EndCapStyleMessage,
    MiterRatioMessage,
    WidthMessage,
)
from ansys.api.edb.v1.pin_pair_model_pb2 import PinPairRlcMessage
from ansys.api.edb.v1.point_3d_data_pb2 import CPos3DMessage, CPos3DTripleMessage, Point3DMessage
from ansys.api.edb.v1.point_data_pb2 import (
    BoxMessage,
    CircleMessage,
    PointMessage,
    PointPairMessage,
    PointsMessage,
)
from ansys.api.edb.v1.point_term_pb2 import PointTermParamsMessage
from ansys.api.edb.v1.polygon_data_pb2 import (
    IntersectionTypeMessage,
    PolygonDataListMessage,
    PolygonDataMessage,
)
from ansys.api.edb.v1.primitive_pb2 import GetPrimitiveHfssPropMessage, PrimitiveTypeMessage
from ansys.api.edb.v1.q3d_simulation_settings_pb2 import Q3DSolutionOrderMessage
from ansys.api.edb.v1.r_tree_pb2 import GeometrySetsMessage, NearestNeighborResponseMessage
from ansys.api.edb.v1.raptor_x_simulation_settings_pb2 import RaptorXSimSettingsOptionsMessage
from ansys.api.edb.v1.rectangle_pb2 import RectangleParametersMessage
from ansys.api.edb.v1.s_parameter_model_pb2 import SParameterModelMessage
from ansys.api.edb.v1.si_wave_dcir_simulation_settings_pb2 import SourceTermsToGroundMessage
from ansys.api.edb.v1.si_wave_psi_simulation_settings_pb2 import (
    ConductorModelingMessage,
    ErrorToleranceMessage,
    ImprovedLossModelMessage,
    SIWavePSIModelTypeMessage,
    SIWavePSISimulationPreferenceMessage,
    SurfaceRoughnessModelMessage,
)
from ansys.api.edb.v1.si_wave_simulation_settings_pb2 import (
    ACDCMergeModeMessage,
    SParamDCBehaviorMessage,
    SParamExtrapolationMessage,
    SParamInterpolationMessage,
)
from ansys.api.edb.v1.simulation_settings_pb2 import DefeatureModelTypeMessage, ViaModelTypeMessage
from ansys.api.edb.v1.simulation_setup_pb2 import SimulationSetupTypeMessage, SweepDataListMessage
from ansys.api.edb.v1.solder_ball_property_pb2 import DiameterMessage
from ansys.api.edb.v1.spice_model_pb2 import SpiceModelMessage
from ansys.api.edb.v1.stackup_layer_pb2 import (
    EtchNetClassMessage,
    HFSSSolverPropertiesMessage,
    RoughnessModelMessage,
    ViaLayerIdsMessage,
)
from ansys.api.edb.v1.structure_3d_pb2 import ClosureMessage
from ansys.api.edb.v1.term_pb2 import TermParamsMessage, TermProductSolversMessage
from ansys.api.edb.v1.text_pb2 import TextDataMessage
from ansys.api.edb.v1.variable_server_pb2 import GetVariableNamesMessage
from ansys.api.edb.v1.via_layer_pb2 import OxideLayersMessage
from ansys.api.edb.v1.voltage_regulator_pb2 import PowerModuleMessage, PowerModulesMessage
from google.protobuf.empty_pb2 import Empty
from google.protobuf.wrappers_pb2 import (
    BoolValue,
    DoubleValue,
    FloatValue,
    Int32Value,
    Int64Value,
    StringValue,
    UInt64Value,
)

rpc_response_map = {
    "ansys.api.edb.v1.ArcDataService": {
        "GetHeight": FloatValue,
        "GetCenter": PointMessage,
        "GetMidpoint": PointMessage,
        "GetRadius": FloatValue,
        "GetBoundingBox": BoxMessage,
        "GetAngle": FloatValue,
        "ClosestPoints": BoxMessage,
    },
    "ansys.api.edb.v1.BoardBendDefService": {
        "Create": EDBObjMessage,
        "GetBoundaryPrim": EDBObjMessage,
        "GetBendMiddle": PointPairMessage,
        "SetBendMiddle": Empty,
        "GetRadius": ValueMessage,
        "SetRadius": Empty,
        "GetAngle": ValueMessage,
        "SetAngle": Empty,
        "GetBentRegions": PolygonDataListMessage,
    },
    "ansys.api.edb.v1.BondwireService": {
        "Create": EDBObjMessage,
        "GetMaterial": StringValue,
        "SetMaterial": Empty,
        "GetType": BondwireTypeMessage,
        "SetType": Empty,
        "GetCrossSectionType": GetCrossSectionTypeMessage,
        "SetCrossSectionType": Empty,
        "GetCrossSectionHeight": ValueMessage,
        "SetCrossSectionHeight": Empty,
        "GetDefinitionName": StringValue,
        "SetDefinitionName": Empty,
        "GetStartElevation": EDBObjPairMessage,
        "SetStartElevation": Empty,
        "GetEndElevation": EDBObjPairMessage,
        "SetEndElevation": Empty,
        "GetTraj": BondwireTrajMessage,
        "SetTraj": Empty,
        "GetWidthValue": ValueMessage,
        "SetWidthValue": Empty,
    },
    "ansys.api.edb.v1.BondwireDefService": {"Delete": Empty, "GetName": StringValue},
    "ansys.api.edb.v1.ApdBondwireDefService": {
        "Create": EDBObjMessage,
        "LoadDefinitionsFromFile": Empty,
        "FindByName": EDBObjMessage,
        "GetParameters": StringValue,
        "SetParameters": Empty,
    },
    "ansys.api.edb.v1.Jedec4BondwireDefService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "GetParameters": ValueMessage,
        "SetParameters": Empty,
    },
    "ansys.api.edb.v1.Jedec5BondwireDefService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "GetParameters": Jedec5BondwireDefParametersMessage,
        "SetParameters": Empty,
    },
    "ansys.api.edb.v1.BundleTerminalService": {
        "Create": EDBObjMessage,
        "Ungroup": Empty,
        "GetTerminals": EDBObjCollectionMessage,
        "StreamTerminals": EDBObjCollectionMessage,
    },
    "ansys.api.edb.v1.CellService": {
        "Create": EDBObjMessage,
        "GetLayout": EDBObjMessage,
        "GetFlattenedLayout": EDBObjMessage,
        "Find": EDBObjMessage,
        "Delete": Empty,
        "GetDatabase": EDBObjMessage,
        "IsFootprint": BoolValue,
        "IsBlackBox": BoolValue,
        "SetBlackBox": Empty,
        "GetSuppressPads": BoolValue,
        "SetSuppressPads": Empty,
        "GetAntiPadsAlwaysOn": BoolValue,
        "SetAntiPadsAlwaysOn": Empty,
        "GetAntiPadsOption": Int64Value,
        "SetAntiPadsOption": Empty,
        "IsSymbolicFootprint": BoolValue,
        "GetName": StringValue,
        "SetName": Empty,
        "GetDesignMode": DesignModeMessage,
        "SetDesignMode": Empty,
        "GetHfssExtentInfo": HfssExtentInfoMessage,
        "SetHfssExtentInfo": Empty,
        "GetTemperatureSettings": TemperatureSettingsMessage,
        "SetTemperatureSettings": Empty,
        "GetTouchstoneExportSettings": TouchstoneExportSettingsMessage,
        "SetTouchstoneExportSettings": Empty,
        "CutOut": EDBObjMessage,
        "GetProductPropertyIds": ProductPropertyIdsMessage,
        "GetProductProperty": StringValue,
        "SetProductProperty": Empty,
        "DeleteSimulationSetup": Empty,
        "GetSimulationSetups": EDBObjCollectionMessage,
        "StreamSimulationSetups": EDBObjCollectionMessage,
        "GenerateAutoHFSSRegions": Empty,
        "GenerateViaSmartBox": PolygonDataListMessage,
        "ApplyTechnology": Empty,
    },
    "ansys.api.edb.v1.CellInstanceService": {
        "Create": EDBObjMessage,
        "CreateWithComponent": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "GetReferenceLayout": EDBObjMessage,
        "GetTermInsts": EDBObjCollectionMessage,
        "StreamTermInsts": EDBObjCollectionMessage,
        "GetIs3DPlacement": BoolValue,
        "Set3DPlacement": Empty,
        "Get3DTransform": EDBObjMessage,
        "Set3DTransform": Empty,
        "GetParameterOverride": ValueMessage,
        "SetParameterOverride": Empty,
    },
    "ansys.api.edb.v1.CircleService": {
        "Create": EDBObjMessage,
        "Render": PolygonDataMessage,
        "GetParameters": CircleParametersMessage,
        "SetParameters": Empty,
    },
    "ansys.api.edb.v1.ComponentDefService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "SetName": Empty,
        "GetName": StringValue,
        "SetFootprintCell": Empty,
        "GetFootprintCell": EDBObjMessage,
        "GetComponentModels": EDBObjCollectionMessage,
        "StreamComponentModels": EDBObjCollectionMessage,
        "GetComponentPins": EDBObjCollectionMessage,
        "StreamComponentPins": EDBObjCollectionMessage,
        "AddComponentModel": Empty,
        "RemoveComponentModel": Empty,
        "ReorderPins": Empty,
        "RemovePin": Empty,
        "GetProductProperty": StringValue,
        "SetProductProperty": Empty,
        "GetProductPropertyIds": ProductPropertyIdsMessage,
    },
    "ansys.api.edb.v1.ComponentGroupService": {
        "Create": EDBObjMessage,
        "GetNumberOfPins": UInt64Value,
        "GetComponentProperty": EDBObjMessage,
        "SetComponentProperty": Empty,
        "GetComponentType": ComponentTypeMessage,
        "SetComponentType": Empty,
        "FindByDef": EDBObjCollectionMessage,
    },
    "ansys.api.edb.v1.ComponentModelService": {
        "SetReferenceFile": Empty,
        "GetReferenceFile": StringValue,
        "FindByName": EDBObjMessage,
        "FindById": EDBObjMessage,
        "GetName": StringValue,
        "GetType": ComponentModelTypeMessage,
        "GetId": UInt64Value,
    },
    "ansys.api.edb.v1.NPortComponentModelService": {"Create": EDBObjMessage},
    "ansys.api.edb.v1.DynamicLinkComponentModelService": {
        "Create": EDBObjMessage,
        "SetDesignName": Empty,
        "GetDesignName": StringValue,
    },
    "ansys.api.edb.v1.ComponentPinService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "SetName": Empty,
        "GetName": StringValue,
        "GetNumber": UInt64Value,
        "GetComponentDef": EDBObjMessage,
    },
    "ansys.api.edb.v1.ComponentPropertyService": {
        "Clone": EDBObjMessage,
        "GetPackageMountingOffset": ValueMessage,
        "SetPackageMountingOffset": Empty,
        "GetPackageDef": EDBObjMessage,
        "SetPackageDef": Empty,
        "GetModel": ComponentPropModelMessage,
        "SetModel": Empty,
    },
    "ansys.api.edb.v1.ConnectableService": {
        "GetObjType": LayoutObjTypeMessage,
        "FindByIdAndType": EDBObjMessage,
        "GetId": EDBInternalIdMessage,
        "GetComponent": EDBObjMessage,
        "GetGroup": EDBObjMessage,
        "SetGroup": Empty,
        "GetNet": EDBObjMessage,
        "SetNet": Empty,
    },
    "ansys.api.edb.v1.DatabaseService": {
        "Create": EDBObjMessage,
        "Open": EDBObjMessage,
        "Delete": Empty,
        "Save": Empty,
        "Close": Empty,
        "IsReadOnly": BoolValue,
        "GetTopCircuits": EDBObjCollectionMessage,
        "StreamTopCircuits": EDBObjCollectionMessage,
        "GetId": Int64Value,
        "FindById": EDBObjMessage,
        "SaveAs": Empty,
        "GetVersionByRelease": StringValue,
        "GetDirectory": StringValue,
        "GetProductProperty": StringValue,
        "SetProductProperty": Empty,
        "GetProductPropertyIds": ProductPropertyIdsMessage,
        "ImportMaterialFromControlFile": Empty,
        "GetVersion": GetVersionMessage,
        "Scale": Empty,
        "GetSource": StringValue,
        "SetSource": Empty,
        "GetSourceVersion": StringValue,
        "SetSourceVersion": Empty,
        "CopyCells": EDBObjCollectionMessage,
        "GetDefinitionObjs": EDBObjCollectionMessage,
        "StreamDefinitionObjs": EDBObjCollectionMessage,
        "TopCircuitCells": EDBObjCollectionMessage,
        "GetCircuits": EDBObjCollectionMessage,
        "StreamCircuits": EDBObjCollectionMessage,
        "GetFootprints": EDBObjCollectionMessage,
        "StreamFootprints": EDBObjCollectionMessage,
        "Encrypt": Empty,
    },
    "ansys.api.edb.v1.DatasetDefService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "GetName": StringValue,
        "SetName": Empty,
        "GetData": PointsMessage,
        "SetData": Empty,
    },
    "ansys.api.edb.v1.DebyeModelService": {
        "Create": EDBObjMessage,
        "GetFrequencyRange": FrequencyRangeMessage,
        "SetFrequencyRange": Empty,
        "GetRelativePermitivityAtHighLowFrequency": FrequencyRangeMessage,
        "SetRelativePermitivityAtHighLowFrequency": Empty,
        "IsRelativePermitivityEnabledAtOpticalFrequency": BoolValue,
        "SetRelativePermitivityEnabledAtOpticalFrequency": Empty,
        "GetRelativePermitivityAtOpticalFrequency": DoubleValue,
        "SetRelativePermitivityAtOpticalFrequency": Empty,
        "UseDCConductivity": BoolValue,
        "SetUseDCConductivity": Empty,
        "GetDCConductivity": DoubleValue,
        "SetDCConductivity": Empty,
        "GetLossTangentAtHighLowFrequency": FrequencyRangeMessage,
        "SetLossTangentAtHighLowFrequency": Empty,
    },
    "ansys.api.edb.v1.DielectricMaterialModelService": {"GetType": Int32Value},
    "ansys.api.edb.v1.DiePropertyService": {
        "Create": EDBObjMessage,
        "Clone": EDBObjMessage,
        "GetDieType": DieTypeMessage,
        "SetDieType": Empty,
        "GetHeight": ValueMessage,
        "SetHeight": Empty,
        "GetOrientation": DieOrientationMessage,
        "SetOrientation": Empty,
    },
    "ansys.api.edb.v1.DifferentialPairService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "GetDifferentialPair": DifferentialPairNetRefsMessage,
        "SetDifferentialPair": Empty,
    },
    "ansys.api.edb.v1.DjordjecvicSarkarModelService": {
        "Create": EDBObjMessage,
        "GetFrequency": DoubleValue,
        "SetFrequency": Empty,
        "GetRelativePermitivityAtFrequency": DoubleValue,
        "SetRelativePermitivityAtFrequency": Empty,
        "GetLossTangentAtFrequency": DoubleValue,
        "SetLossTangentAtFrequency": Empty,
        "GetHighFrequencyCorner": DoubleValue,
        "SetHighFrequencyCorner": Empty,
        "UseDCRelativePermitivity": BoolValue,
        "SetUseDCRelativePermitivity": Empty,
        "GetDCRelativePermitivity": DoubleValue,
        "SetDCRelativePermitivity": Empty,
        "GetDCConductivity": DoubleValue,
        "SetDCConductivity": Empty,
    },
    "ansys.api.edb.v1.EDBErrorManagerService": {"GetErrors": EDBErrorMessages},
    "ansys.api.edb.v1.EdgeService": {
        "GetEdgeType": EdgeTypeMessage,
        "Create": EDBObjMessage,
        "GetParameters": EdgeParamsMessage,
    },
    "ansys.api.edb.v1.EdgeTerminalService": {
        "Create": EDBObjMessage,
        "GetEdges": EDBObjCollectionMessage,
        "StreamEdges": EDBObjCollectionMessage,
        "SetEdges": Empty,
    },
    "ansys.api.edb.v1.ExtendedNetService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "RemoveNet": Empty,
        "AddNet": Empty,
        "RemoveAllNets": Empty,
    },
    "ansys.api.edb.v1.GroupService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "AddMember": Empty,
        "RemoveMember": Empty,
        "Ungroup": Empty,
        "GetMembers": EDBObjCollectionMessage,
        "StreamMembers": EDBObjCollectionMessage,
        "GetGroupType": GroupTypeMessage,
    },
    "ansys.api.edb.v1.HFSSPIGeneralSettingsService": {
        "GetHFSSPIModelType": HFSSPIModelTypeMessage,
        "SetHFSSPIModelType": Empty,
        "GetUseAutoMeshRegion": BoolValue,
        "SetUseAutoMeshRegion": Empty,
        "GetUseMeshRegion": BoolValue,
        "SetUseMeshRegion": Empty,
        "GetMeshRegionName": StringValue,
        "SetMeshRegionName": Empty,
    },
    "ansys.api.edb.v1.HFSSPIAdvancedSettingsService": {
        "GetICModeAutoResolution": BoolValue,
        "SetICModeAutoResolution": Empty,
        "GetICModeLength": StringValue,
        "SetICModeLength": Empty,
        "GetSmallPlaneArea": StringValue,
        "SetSmallPlaneArea": Empty,
        "GetZeroMetalLayerThickness": StringValue,
        "SetZeroMetalLayerThickness": Empty,
    },
    "ansys.api.edb.v1.HFSSPISolverSettingsService": {
        "GetEnhancedLowFrequencyAccuracy": BoolValue,
        "SetEnhancedLowFrequencyAccuracy": Empty,
        "GetViaAreaCutoffCircElems": StringValue,
        "SetViaAreaCutoffCircElems": Empty,
    },
    "ansys.api.edb.v1.HFSSGeneralSettingsService": {
        "GetSingleFrequencyAdaptiveSolution": SingleFrequencyAdaptiveSolutionMessage,
        "SetSingleFrequencyAdaptiveSolution": Empty,
        "GetMultiFrequencyAdaptiveSolution": MultiFrequencyAdaptiveSolutionMessage,
        "SetMultiFrequencyAdaptiveSolution": Empty,
        "GetBroadbandFrequencyAdaptiveSolution": BroadbandFrequencyAdaptiveSolutionMessage,
        "SetBroadbandFrequencyAdaptiveSolution": Empty,
        "GetSaveFieldsFlag": BoolValue,
        "SetSaveFieldsFlag": Empty,
        "GetUseMeshRegion": BoolValue,
        "SetUseMeshRegion": Empty,
        "GetMeshRegionName": StringValue,
        "SetMeshRegionName": Empty,
        "GetUseParallelRefinement": BoolValue,
        "SetUseParallelRefinement": Empty,
        "GetAdaptType": AdaptTypeMessage,
        "SetAdaptType": Empty,
        "GetSaveRadFieldsOnlyFlag": BoolValue,
        "SetSaveRadFieldsOnlyFlag": Empty,
    },
    "ansys.api.edb.v1.HFSSOptionsSettingsService": {
        "GetMaxRefinementPerPass": UInt64Value,
        "SetMaxRefinementPerPass": Empty,
        "GetMinPasses": UInt64Value,
        "SetMinPasses": Empty,
        "GetMinConvergedPasses": UInt64Value,
        "SetMinConvergedPasses": Empty,
        "GetUseMaxRefinement": BoolValue,
        "SetUseMaxRefinement": Empty,
        "GetBasisFunctionOrder": BasisFunctionOrderMessage,
        "SetBasisFunctionOrder": Empty,
        "GetSolveInsideMetalBasis": BasisFunctionOrderMessage,
        "SetSolveInsideMetalBasis": Empty,
        "GetSolverTypeOrder": SolverTypeMessage,
        "SetSolverTypeOrder": Empty,
        "GetRelativeResidual": DoubleValue,
        "SetRelativeResidual": Empty,
        "GetUseShellElements": BoolValue,
        "SetUseShellElements": Empty,
        "GetEnhancedLowFrequencyAccuracy": BoolValue,
        "SetEnhancedLowFrequencyAccuracy": Empty,
    },
    "ansys.api.edb.v1.HFSSAdvancedSettingsService": {
        "GetICModeAutoResolution": BoolValue,
        "SetICModeAutoResolution": Empty,
        "GetICModeLength": StringValue,
        "SetICModeLength": Empty,
    },
    "ansys.api.edb.v1.HFSSAdvancedMeshingSettingsService": {
        "GetLayerAlignment": StringValue,
        "SetLayerAlignment": Empty,
    },
    "ansys.api.edb.v1.HFSSSolverSettingsService": {
        "GetMaxDeltaZ0": DoubleValue,
        "SetMaxDeltaZ0": Empty,
        "GetSetTrianglesForWaveport": BoolValue,
        "SetSetTrianglesForWaveport": Empty,
        "GetMinTrianglesForWavePort": UInt64Value,
        "SetMinTrianglesForWavePort": Empty,
        "GetMaxTrianglesForWavePort": UInt64Value,
        "SetMaxTrianglesForWavePort": Empty,
        "GetIntraPlaneCouplingEnabled": BoolValue,
        "SetIntraPlaneCouplingEnabled": Empty,
    },
    "ansys.api.edb.v1.DCRSettingsService": {
        "GetMaxPasses": UInt64Value,
        "SetMaxPasses": Empty,
        "GetMinPasses": UInt64Value,
        "SetMinPasses": Empty,
        "GetMinConvergedPasses": UInt64Value,
        "SetMinConvergedPasses": Empty,
        "GetPercentError": DoubleValue,
        "SetPercentError": Empty,
        "GetPercentRefinementPerPass": DoubleValue,
        "SetPercentRefinementPerPass": Empty,
    },
    "ansys.api.edb.v1.HfssSimulationSetupService": {
        "GetMeshOperations": MeshOperationsMessage,
        "SetMeshOperations": Empty,
    },
    "ansys.api.edb.v1.HierarchyObjectService": {
        "GetTransform": EDBObjMessage,
        "SetTransform": Empty,
        "GetName": StringValue,
        "SetName": Empty,
        "GetComponent": EDBObjMessage,
        "GetPlacementLayer": EDBObjMessage,
        "SetPlacementLayer": Empty,
        "GetLocation": PointMessage,
        "SetLocation": Empty,
        "GetSolveIndependentPreference": BoolValue,
        "SetSolveIndependentPreference": Empty,
    },
    "ansys.api.edb.v1.ICComponentPropertyService": {
        "Create": EDBObjMessage,
        "SetSolderBallProperty": Empty,
        "GetSolderBallProperty": EDBObjMessage,
        "SetDieProperty": Empty,
        "GetDieProperty": EDBObjMessage,
        "SetPortProperty": Empty,
        "GetPortProperty": EDBObjMessage,
    },
    "ansys.api.edb.v1.InstArrayService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "GetOrig": PointMessage,
        "GetXAxis": PointMessage,
        "SetXAxis": Empty,
        "GetYAxis": PointMessage,
        "SetYAxis": Empty,
        "GetXCount": ValueMessage,
        "SetXCount": Empty,
        "GetYCount": ValueMessage,
        "SetYCount": Empty,
        "Decompose": Empty,
    },
    "ansys.api.edb.v1.IOComponentPropertyService": {
        "Create": EDBObjMessage,
        "SetSolderBallProperty": Empty,
        "GetSolderBallProperty": EDBObjMessage,
        "SetPortProperty": Empty,
        "GetPortProperty": EDBObjMessage,
    },
    "ansys.api.edb.v1.IOManagerService": {
        "FlushBuffer": ResolvedFuturesMessage,
        "FlushBufferStream": ResolvedFuturesMessage,
        "FlushBufferNoFutureResolutionStream": Empty,
        "RefreshCache": EDBObjCollectionMessage,
        "EnableCache": Empty,
    },
    "ansys.api.edb.v1.LayerService": {
        "Create": EDBObjMessage,
        "GetLayerType": LayerTypeMessage,
        "SetLayerType": Empty,
        "IsViaLayer": BoolValue,
        "GetName": StringValue,
        "SetName": Empty,
        "Clone": EDBObjMessage,
        "GetLayerId": Int64Value,
        "GetTopBottomAssociation": TopBottomAssociationMessage,
        "SetTopBottomAssociation": Empty,
        "GetColor": UInt64Value,
        "SetColor": Empty,
        "GetVisibilityMask": UInt64Value,
        "SetVisibilityMask": Empty,
        "GetLocked": BoolValue,
        "SetLocked": Empty,
        "GetTransparency": UInt64Value,
        "SetTransparency": Empty,
        "GetDrawOverride": DrawOverrideMessage,
        "SetDrawOverride": Empty,
        "GetProductProperty": StringValue,
        "SetProductProperty": Empty,
        "GetProductPropertyIds": ProductPropertyIdsMessage,
        "IsInZone": BoolValue,
        "SetIsInZone": Empty,
        "GetZones": ZonesMessage,
        "GetZone": Int64Value,
    },
    "ansys.api.edb.v1.LayerCollectionService": {
        "Create": EDBObjMessage,
        "Clone": EDBObjMessage,
        "GetMode": LayerCollectionModeMessage,
        "SetMode": Empty,
        "AddLayers": Empty,
        "ImportFromControlFile": Empty,
        "AddLayer": EDBObjMessage,
        "IsValid": BoolValue,
        "FindByName": EDBObjMessage,
        "GetTopBottomStackupLayers": TopBottomStackupLayersMessage,
        "GetLayers": EDBObjCollectionMessage,
        "StreamLayers": EDBObjCollectionMessage,
        "GetProductProperty": StringValue,
        "SetProductProperty": Empty,
        "GetProductPropertyIds": ProductPropertyIdsMessage,
        "MergeDielectrics": EDBObjMessage,
        "GetZoneIds": ZonesMessage,
        "GetZoneName": StringValue,
        "SetZoneName": Empty,
        "InsertZone": Int64Value,
        "RemoveZone": Empty,
        "SimplifyDielectricsForPhi": EDBObjCollectionMessage,
        "AddZoneToLayer": Empty,
    },
    "ansys.api.edb.v1.LayerMapService": {
        "Create": EDBObjMessage,
        "Clear": Empty,
        "SetMapping": Empty,
        "GetMappingForward": ProductPropertyIdsMessage,
        "GetMappingBackward": ProductPropertyIdsMessage,
    },
    "ansys.api.edb.v1.LayoutService": {
        "GetCell": EDBObjMessage,
        "GetLayerCollection": EDBObjMessage,
        "SetLayerCollection": Empty,
        "GetItems": EDBObjCollectionMessage,
        "StreamItems": EDBObjCollectionMessage,
        "GetExpandedExtentFromNets": PolygonDataMessage,
        "ConvertPrimitivesToVias": Empty,
        "ArePortReferenceTerminalsConnected": LayoutPortRefTerminalsConnectedMessage,
        "GetZonePrimitives": EDBObjCollectionMessage,
        "StreamZonePrimitives": EDBObjCollectionMessage,
        "SetFixedZonePrimitives": Empty,
        "GetFixedZonePrimitive": EDBObjMessage,
        "GetBoardBendDefs": EDBObjCollectionMessage,
        "StreamBoardBendDefs": EDBObjCollectionMessage,
        "SynchronizeBendManager": Empty,
        "GetLayoutInstance": EDBObjMessage,
        "ReconstructArcs": Empty,
        "UnitePrimitives": Empty,
        "GroupVias": Empty,
        "SnapVias": Empty,
        "SnapPrimitives": Empty,
        "CreateMeshRegion": Empty,
        "CompressPrimitives": Empty,
        "FlattenCellInstances": Empty,
        "RemoveHolesExceptOnCriticalNets": Empty,
        "RemoveUnusedConductingLayers": Empty,
    },
    "ansys.api.edb.v1.LayoutComponentService": {
        "ExportLayoutComponent": Empty,
        "ImportLayoutComponent": EDBObjMessage,
        "GetCellInstance": EDBObjMessage,
    },
    "ansys.api.edb.v1.LayoutInstanceService": {
        "Refresh": Empty,
        "QueryLayoutObjInstances": LayoutObjInstancesQueryResultsMessage,
        "StreamLayoutObjInstancesQuery": LayoutObjInstancesQueryResultsMessage,
        "GetLayoutObjInstanceInContext": EDBObjMessage,
        "GetConnectedObjects": EDBObjCollectionMessage,
        "StreamConnectedObjects": EDBObjCollectionMessage,
    },
    "ansys.api.edb.v1.LayoutInstanceContextService": {
        "GetLayout": EDBObjMessage,
        "GetBBox": BoxMessage,
        "IsTopOrBlackBox": BoolValue,
        "GetTopOrBlackBox": EDBObjMessage,
        "GetPlacementElevation": DoubleValue,
        "Is3DPlacement": BoolValue,
        "GetTransformation": EDBObjMessage,
        "GetTransformationBetweenContexts": EDBObjMessage,
    },
    "ansys.api.edb.v1.LayoutObjService": {
        "GetLayout": EDBObjMessage,
        "Delete": Empty,
        "GetProductProperty": StringValue,
        "SetProductProperty": Empty,
        "GetProductPropertyIds": ProductPropertyIdsMessage,
    },
    "ansys.api.edb.v1.LayoutObjInstanceService": {
        "GetLayers": EDBObjCollectionMessage,
        "StreamLayers": EDBObjCollectionMessage,
        "GetGeometries": FetchedLayoutObjInstanceGeometriesMessage,
        "GetContext": StringsMessage,
        "GetLayoutInstanceContext": EDBObjMessage,
        "GetLayoutObj": EDBObjMessage,
        "GetBBox": BoxMessage,
    },
    "ansys.api.edb.v1.LayoutObjInstance2DGeometryService": {
        "IsNegative": BoolValue,
        "GetPolygonData": PolygonDataMessage,
    },
    "ansys.api.edb.v1.LayoutObjInstance3DGeometryService": {
        "GetTesselationData": TesselationDataMessage
    },
    "ansys.api.edb.v1.LayoutObjInstanceGeometryService": {
        "GetMaterial": StringValue,
        "GetColor": Int64Value,
    },
    "ansys.api.edb.v1.MaterialDefService": {
        "Create": EDBObjMessage,
        "SetProperty": Empty,
        "FindByName": EDBObjMessage,
        "Delete": Empty,
        "GetProperty": ValueMessage,
        "GetAllProperties": MaterialDefGetAllPropertiesMessage,
        "RemoveProperty": Empty,
        "GetName": StringValue,
        "GetDielectricMaterialModel": EDBObjMessage,
        "SetDielectricMaterialModel": Empty,
        "GetDimensions": MaterialDefGetDimensionMessage,
        "SetThermalModifier": Empty,
        "SetAnisotropicThermalModifier": Empty,
        "GetThermalModifier": EDBObjMessage,
        "GetAnisotropicThermalModifier": EDBObjMessage,
    },
    "ansys.api.edb.v1.MaterialPropertyThermalModifierService": {
        "Create": EDBObjMessage,
        "GetQuadraticModelParams": MaterialPropertyThermalModifierParamsMessage,
        "GetThermalModifierExpression": ValueMessage,
    },
    "ansys.api.edb.v1.McadModelService": {
        "CreateStride": EDBObjMessage,
        "CreateHfss": EDBObjMessage,
        "Create3dComp": EDBObjMessage,
        "IsMcad": BoolValue,
        "IsMcadStride": BoolValue,
        "IsMcadHfss": BoolValue,
        "IsMcad3dComp": BoolValue,
        "GetOrigin": Point3DMessage,
        "SetOrigin": Empty,
        "GetRotation": McadModelRotationMessage,
        "SetRotation": Empty,
        "GetScale": DoubleValue,
        "SetScale": Empty,
        "GetMaterial": StringValue,
        "SetMaterial": Empty,
        "GetVisible": BoolValue,
        "SetVisible": Empty,
        "GetModeled": BoolValue,
        "SetModeled": Empty,
        "GetCellInst": EDBObjMessage,
        "GetPartCount": Int64Value,
        "GetPartName": StringValue,
        "GetPartIndex": Int64Value,
        "GetModelName": StringValue,
        "GetDesignName": StringValue,
    },
    "ansys.api.edb.v1.ModelService": {"Clone": EDBObjMessage},
    "ansys.api.edb.v1.MultipoleDebyeModelService": {
        "Create": EDBObjMessage,
        "GetParameters": MultipoleDebyeModelGetParams,
        "SetParameters": Empty,
    },
    "ansys.api.edb.v1.NetService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "GetName": StringValue,
        "SetName": Empty,
        "GetIsPowerGround": BoolValue,
        "SetIsPowerGround": Empty,
        "GetLayoutObjects": EDBObjCollectionMessage,
        "StreamLayoutObjects": EDBObjCollectionMessage,
    },
    "ansys.api.edb.v1.NetlistModelService": {
        "Create": EDBObjMessage,
        "GetNetlist": StringValue,
        "SetNetlist": Empty,
    },
    "ansys.api.edb.v1.NetClassService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "GetName": StringValue,
        "SetName": Empty,
        "GetDescription": StringValue,
        "SetDescription": Empty,
        "IsPowerGround": BoolValue,
        "GetNets": EDBObjCollectionMessage,
        "StreamNets": EDBObjCollectionMessage,
        "AddNet": Empty,
        "RemoveNet": Empty,
        "ContainsNet": BoolValue,
    },
    "ansys.api.edb.v1.PackageDefService": {
        "Create": EDBObjMessage,
        "Delete": Empty,
        "FindByName": EDBObjMessage,
        "FindByEDBUId": EDBObjMessage,
        "GetName": StringValue,
        "SetName": Empty,
        "GetExteriorBoundary": PolygonDataMessage,
        "SetExteriorBoundary": Empty,
        "GetHeight": ValueMessage,
        "SetHeight": Empty,
        "GetOperatingPower": ValueMessage,
        "SetOperatingPower": Empty,
        "GetMaximumPower": ValueMessage,
        "SetMaximumPower": Empty,
        "GetTherm_Cond": ValueMessage,
        "SetTherm_Cond": Empty,
        "GetTheta_JB": ValueMessage,
        "SetTheta_JB": Empty,
        "GetTheta_JC": ValueMessage,
        "SetTheta_JC": Empty,
        "GetHeatSink": HeatSinkMessage,
        "SetHeatSink": Empty,
        "GetProductProperty": StringValue,
        "SetProductProperty": Empty,
        "GetProductPropertyIds": ProductPropertyIdsMessage,
    },
    "ansys.api.edb.v1.PadstackDefService": {
        "Create": EDBObjMessage,
        "Delete": Empty,
        "FindByName": EDBObjMessage,
        "GetName": StringValue,
        "GetData": EDBObjMessage,
        "SetData": Empty,
    },
    "ansys.api.edb.v1.PadstackDefDataService": {
        "Create": EDBObjMessage,
        "GetMaterial": StringValue,
        "SetMaterial": Empty,
        "GetLayerNames": PadstackDefDataGetLayerNamesMessage,
        "GetLayerIds": PadstackDefDataGetLayerIdsMessage,
        "AddLayers": Empty,
        "GetPadParameters": PadstackDefDataPadParametersMessage,
        "SetPadParameters": Empty,
        "GetHoleRange": PadstackDefDataPadstackHoleRangeMessage,
        "SetHoleRange": Empty,
        "GetPlatingPercentage": ValueMessage,
        "SetPlatingPercentage": Empty,
        "GetSolderBallShape": PadstackDefDataSolderballShapeMessage,
        "SetSolderBallShape": Empty,
        "GetSolderBallPlacement": PadstackDefDataSolderballPlacementMessage,
        "SetSolderBallPlacement": Empty,
        "GetSolderBallParam": PadstackDefDataGetSolderBallParamMessage,
        "SetSolderBallParam": Empty,
        "GetSolderBallMaterial": StringValue,
        "SetSolderBallMaterial": Empty,
        "GetConnectionPt": PadstackDefDataConnectionPtMessage,
        "SetConnectionPt": Empty,
    },
    "ansys.api.edb.v1.PadstackInstanceService": {
        "Create": EDBObjMessage,
        "GetPadstackDef": EDBObjMessage,
        "GetName": StringValue,
        "SetName": Empty,
        "GetPositionAndRotation": PadstackInstPositionAndRotationMessage,
        "SetPositionAndRotation": Empty,
        "GetLayerRange": PadstackInstLayerRangeMessage,
        "SetLayerRange": Empty,
        "GetSolderBallLayer": EDBObjMessage,
        "SetSolderBallLayer": Empty,
        "GetLayerMap": EDBObjMessage,
        "SetLayerMap": Empty,
        "GetHoleOverrides": PadstackInstHoleOverridesMessage,
        "SetHoleOverrides": Empty,
        "GetIsLayoutPin": BoolValue,
        "SetIsLayoutPin": Empty,
        "GetBackDrillType": PadstackInstBackDrillTypeMessage,
        "GetBackDrillByLayer": PadstackInstBackDrillByLayerMessage,
        "GetBackDrillByDepth": PadstackInstBackDrillByDepthMessage,
        "SetBackDrillByLayer": Empty,
        "SetBackDrillByDepth": Empty,
        "GetPadstackInstanceTerminal": EDBObjMessage,
        "IsInPinGroup": BoolValue,
        "GetPinGroups": EDBObjCollectionMessage,
        "StreamPinGroups": EDBObjCollectionMessage,
    },
    "ansys.api.edb.v1.PadstackInstanceTerminalService": {
        "Create": EDBObjMessage,
        "GetParameters": PadstackInstTermParamsMessage,
        "SetParameters": Empty,
    },
    "ansys.api.edb.v1.PathService": {
        "Create": EDBObjMessage,
        "Render": PolygonDataMessage,
        "GetPolygonData": PolygonDataMessage,
        "GetCenterLine": PolygonDataMessage,
        "SetCenterLine": Empty,
        "GetEndCapStyle": EndCapStyleMessage,
        "SetEndCapStyle": Empty,
        "GetClipInfo": ClipInfoMessage,
        "SetClipInfo": Empty,
        "GetCornerStyle": CornerStyleMessage,
        "SetCornerStyle": Empty,
        "GetWidth": WidthMessage,
        "SetWidth": Empty,
        "GetMiterRatio": MiterRatioMessage,
        "SetMiterRatio": Empty,
    },
    "ansys.api.edb.v1.PinGroupService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "GetUniqueName": StringValue,
        "GetName": StringValue,
        "AddPins": Empty,
        "RemovePins": Empty,
        "GetPins": EDBObjCollectionMessage,
        "StreamPins": EDBObjCollectionMessage,
        "GetPinGroupTerminal": EDBObjMessage,
    },
    "ansys.api.edb.v1.PinGroupTerminalService": {
        "Create": EDBObjMessage,
        "GetPinGroup": EDBObjMessage,
        "SetPinGroup": Empty,
        "GetLayer": EDBObjMessage,
        "SetLayer": Empty,
    },
    "ansys.api.edb.v1.PinPairModelService": {
        "Create": EDBObjMessage,
        "GetRlc": PinPairRlcMessage,
        "SetRlc": Empty,
        "DeleteRlc": Empty,
        "GetPinPairs": StringPairsMessage,
    },
    "ansys.api.edb.v1.PointDataService": {
        "Rotate": PointMessage,
        "ClosestPoint": PointMessage,
        "Distance": FloatValue,
    },
    "ansys.api.edb.v1.PointTerminalService": {
        "Create": EDBObjMessage,
        "GetParameters": PointTermParamsMessage,
        "SetParameters": Empty,
    },
    "ansys.api.edb.v1.PolygonService": {
        "Create": EDBObjMessage,
        "GetPolygonData": PolygonDataMessage,
        "SetPolygonData": Empty,
    },
    "ansys.api.edb.v1.PolygonDataService": {
        "GetNormalizedPoints": PointsMessage,
        "IsCircle": BoolValue,
        "IsBox": BoolValue,
        "HasSelfIntersections": BoolValue,
        "RemoveSelfIntersections": PolygonDataListMessage,
        "IsConvex": BoolValue,
        "GetArea": FloatValue,
        "Transform": PolygonDataMessage,
        "GetBBox": BoxMessage,
        "GetStreamedBBox": BoxMessage,
        "GetConvexHull": PolygonDataMessage,
        "RemoveArcs": PolygonDataMessage,
        "Defeature": PolygonDataMessage,
        "GetBoundingCircle": CircleMessage,
        "IsInside": BoolValue,
        "CircleIntersectsPolygon": BoolValue,
        "GetIntersectionType": IntersectionTypeMessage,
        "GetClosestPoints": PointsMessage,
        "GetUnion": PolygonDataListMessage,
        "GetIntersection": PolygonDataListMessage,
        "Subtract": PolygonDataListMessage,
        "Xor": PolygonDataListMessage,
        "Expand": PolygonDataListMessage,
        "Get2DAlphaShape": PolygonDataListMessage,
    },
    "ansys.api.edb.v1.PortPropertyService": {
        "Create": EDBObjMessage,
        "Clone": EDBObjMessage,
        "GetReferenceHeight": ValueMessage,
        "SetReferenceHeight": Empty,
        "GetReferenceSizeAuto": BoolValue,
        "SetReferenceSizeAuto": Empty,
        "GetReferenceSize": ValuePairMessage,
        "SetReferenceSize": Empty,
    },
    "ansys.api.edb.v1.PrimitiveService": {
        "GetPrimitiveType": PrimitiveTypeMessage,
        "AddVoid": Empty,
        "SetHfssProp": Empty,
        "GetLayer": EDBObjMessage,
        "SetLayer": Empty,
        "GetIsNegative": BoolValue,
        "SetIsNegative": Empty,
        "IsVoid": BoolValue,
        "HasVoids": BoolValue,
        "Voids": EDBObjCollectionMessage,
        "StreamVoids": EDBObjCollectionMessage,
        "GetOwner": EDBObjMessage,
        "IsParameterized": BoolValue,
        "GetHfssProp": GetPrimitiveHfssPropMessage,
        "RemoveHfssProp": Empty,
        "IsZonePrimitive": BoolValue,
        "MakeZonePrimitive": Empty,
    },
    "ansys.api.edb.v1.PrimitiveInstanceCollectionService": {
        "Create": EDBObjMessage,
        "GetGeometry": PolygonDataMessage,
        "SetGeometry": Empty,
        "GetPositions": PointsMessage,
        "SetPositions": Empty,
        "GetInstantiatedGeometry": PolygonDataListMessage,
        "Decompose": Empty,
    },
    "ansys.api.edb.v1.Q3DGeneralSettingsService": {
        "GetSolutionFrequency": StringValue,
        "SetSolutionFrequency": Empty,
        "GetDoDC": BoolValue,
        "SetDoDC": Empty,
        "GetDoDCResOnly": BoolValue,
        "SetDoDCResOnly": Empty,
        "GetDoCG": BoolValue,
        "SetDoCG": Empty,
        "GetDoAC": BoolValue,
        "SetDoAC": Empty,
        "GetSaveFields": BoolValue,
        "SetSaveFields": Empty,
    },
    "ansys.api.edb.v1.Q3DSettingsService": {
        "GetMaxPasses": UInt64Value,
        "SetMaxPasses": Empty,
        "GetMinPasses": UInt64Value,
        "SetMinPasses": Empty,
        "GetMinConvergedPasses": UInt64Value,
        "SetMinConvergedPasses": Empty,
        "GetPercentError": DoubleValue,
        "SetPercentError": Empty,
        "GetMaxRefinePerPass": DoubleValue,
        "SetMaxRefinePerPass": Empty,
    },
    "ansys.api.edb.v1.Q3DDCRLSettingsService": {
        "GetSolutionOrder": Q3DSolutionOrderMessage,
        "SetSolutionOrder": Empty,
    },
    "ansys.api.edb.v1.Q3DCGSettingsService": {
        "GetAutoIncrSolOrder": BoolValue,
        "SetAutoIncrSolOrder": Empty,
        "GetSolutionOrder": Q3DSolutionOrderMessage,
        "SetSolutionOrder": Empty,
        "GetSolverType": SolverTypeMessage,
        "SetSolverType": Empty,
        "GetCompressionTol": DoubleValue,
        "SetCompressionTol": Empty,
    },
    "ansys.api.edb.v1.Q3DAdvancedSettingsService": {
        "GetICModeAutoResolution": BoolValue,
        "SetICModeAutoResolution": Empty,
        "GetICModeLength": StringValue,
        "SetICModeLength": Empty,
    },
    "ansys.api.edb.v1.Q3DAdvancedMeshingSettingsService": {
        "GetLayerAlignment": StringValue,
        "SetLayerAlignment": Empty,
    },
    "ansys.api.edb.v1.RaptorXGeneralSettingsService": {
        "GetUseGoldEMSolver": BoolValue,
        "SetUseGoldEMSolver": Empty,
        "GetMaxFrequency": StringValue,
        "SetMaxFrequency": Empty,
        "GetGlobalTemperature": DoubleValue,
        "SetGlobalTemperature": Empty,
        "GetSaveNetlist": BoolValue,
        "SetSaveNetlist": Empty,
        "GetNetlistExportSpectre": BoolValue,
        "SetNetlistExportSpectre": Empty,
        "GetSaveRFM": BoolValue,
        "SetSaveRFM": Empty,
    },
    "ansys.api.edb.v1.RaptorXAdvancedSettingsService": {
        "GetUseMeshFrequency": BoolValue,
        "SetUseMeshFrequency": Empty,
        "GetMeshFrequency": StringValue,
        "SetMeshFrequency": Empty,
        "GetUseEdgeMesh": BoolValue,
        "SetUseEdgeMesh": Empty,
        "GetEdgeMesh": StringValue,
        "SetEdgeMesh": Empty,
        "GetUseCellsPerWavelength": BoolValue,
        "SetUseCellsPerWavelength": Empty,
        "GetCellsPerWavelength": UInt64Value,
        "SetCellsPerWavelength": Empty,
        "GetUsePlaneProjectionFactor": BoolValue,
        "SetUsePlaneProjectionFactor": Empty,
        "GetPlaneProjectionFactor": DoubleValue,
        "SetPlaneProjectionFactor": Empty,
        "GetUseRelaxedZAxis": BoolValue,
        "SetUseRelaxedZAxis": Empty,
        "GetUseEliminateSlitPerHoles": BoolValue,
        "SetUseEliminateSlitPerHoles": Empty,
        "GetEliminateSlitPerHoles": DoubleValue,
        "SetEliminateSlitPerHoles": Empty,
        "GetUseAutoRemovalSliverPoly": BoolValue,
        "SetUseAutoRemovalSliverPoly": Empty,
        "GetAutoRemovalSliverPoly": DoubleValue,
        "SetAutoRemovalSliverPoly": Empty,
        "GetUseAccelerateViaExtraction": BoolValue,
        "SetUseAccelerateViaExtraction": Empty,
        "GetUseEnableSubstrateNetworkExtraction": BoolValue,
        "SetUseEnableSubstrateNetworkExtraction": Empty,
        "GetUseLDE": BoolValue,
        "SetUseLDE": Empty,
        "GetUseExtractFloatingMetalsDummy": BoolValue,
        "SetUseExtractFloatingMetalsDummy": Empty,
        "GetUseExtractFloatingMetalsFloating": BoolValue,
        "SetUseExtractFloatingMetalsFloating": Empty,
        "GetUseEnableEtchTransform": BoolValue,
        "SetUseEnableEtchTransform": Empty,
        "GetUseEnableHybridExtraction": BoolValue,
        "SetUseEnableHybridExtraction": Empty,
        "GetUseEnableAdvancedCapEffects": BoolValue,
        "SetUseEnableAdvancedCapEffects": Empty,
        "GetUseOverrideShrinkFac": BoolValue,
        "SetUseOverrideShrinkFac": Empty,
        "GetOverrideShrinkFac": DoubleValue,
        "SetOverrideShrinkFac": Empty,
        "GetAdvancedOptions": RaptorXSimSettingsOptionsMessage,
        "SetAdvancedOptions": Empty,
        "GetNetSettingsOptions": RaptorXSimSettingsOptionsMessage,
        "SetNetSettingsOptions": Empty,
    },
    "ansys.api.edb.v1.RectangleService": {
        "Create": EDBObjMessage,
        "GetParameters": RectangleParametersMessage,
        "SetParameters": Empty,
        "Render": PolygonDataMessage,
        "GetPolygonData": PolygonDataMessage,
    },
    "ansys.api.edb.v1.RLCComponentPropertyService": {
        "Create": EDBObjMessage,
        "GetEnabled": BoolValue,
        "SetEnabled": Empty,
    },
    "ansys.api.edb.v1.RTreeService": {
        "Create": EDBObjMessage,
        "GetExtent": BoxMessage,
        "InsertIntObject": Empty,
        "DeleteIntObject": Empty,
        "Empty": BoolValue,
        "Search": IntegersPropertyMessage,
        "NearestNeighbor": NearestNeighborResponseMessage,
        "TouchingGeometry": IntegersPropertyMessage,
        "ConnectedGeometry": IntegersPropertyMessage,
        "GetConnectedGeometrySets": GeometrySetsMessage,
        "IncrementVisit": Empty,
        "IsVisited": BoolValue,
        "Visit": Empty,
        "GetVisit": Int64Value,
    },
    "ansys.api.edb.v1.SimulationSettingsService": {"GetEnabled": BoolValue, "SetEnabled": Empty},
    "ansys.api.edb.v1.SettingsOptionsService": {
        "GetDoLamdaRefineFlag": BoolValue,
        "SetDoLamdaRefineFlag": Empty,
        "GetLamdaTarget": DoubleValue,
        "SetLamdaTarget": Empty,
        "GetMeshSizefactor": DoubleValue,
        "SetMeshSizefactor": Empty,
        "GetUseDefaultLamda": BoolValue,
        "SetUseDefaultLamda": Empty,
    },
    "ansys.api.edb.v1.AdvancedSettingsService": {
        "GetUnionPolygons": BoolValue,
        "SetUnionPolygons": Empty,
        "GetRemoveFloatingGeometry": BoolValue,
        "SetRemoveFloatingGeometry": Empty,
        "GetHealingOption": Int64Value,
        "SetHealingOption": Empty,
        "GetSmallVoidArea": DoubleValue,
        "SetSmallVoidArea": Empty,
        "GetUseDefeature": BoolValue,
        "SetUseDefeature": Empty,
        "GetUseDefeatureAbsoluteLength": BoolValue,
        "SetUseDefeatureAbsoluteLength": Empty,
        "GetDefeatureAbsoluteLength": StringValue,
        "SetDefeatureAbsoluteLength": Empty,
        "GetDefeatureRatio": DoubleValue,
        "SetDefeatureRatio": Empty,
        "GetViaModelType": ViaModelTypeMessage,
        "SetViaModelType": Empty,
        "GetNumViaSides": UInt64Value,
        "SetNumViaSides": Empty,
        "GetViaDensity": DoubleValue,
        "SetViaDensity": Empty,
        "GetViaMaterial": StringValue,
        "SetViaMaterial": Empty,
        "GetMeshForViaPlating": BoolValue,
        "SetMeshForViaPlating": Empty,
        "GetModelType": DefeatureModelTypeMessage,
        "SetModelType": Empty,
    },
    "ansys.api.edb.v1.AdvancedMeshingSettingsService": {
        "GetArcStepSize": StringValue,
        "SetArcStepSize": Empty,
        "GetCircleStartAzimuth": StringValue,
        "SetCircleStartAzimuth": Empty,
        "GetMaxNumArcPoints": UInt64Value,
        "SetMaxNumArcPoints": Empty,
        "GetUseArcChordErrorApprox": BoolValue,
        "SetUseArcChordErrorApprox": Empty,
        "GetArcChordErrorApprox": StringValue,
        "SetArcChordErrorApprox": Empty,
    },
    "ansys.api.edb.v1.SolverSettingsService": {
        "GetThinSignalLayerThreshold": StringValue,
        "SetThinSignalLayerThreshold": Empty,
        "GetThinDielectricLayerThreshold": StringValue,
        "SetThinDielectricLayerThreshold": Empty,
    },
    "ansys.api.edb.v1.SimulationSetupService": {
        "Create": EDBObjMessage,
        "GetName": StringValue,
        "SetName": Empty,
        "GetPosition": UInt64Value,
        "SetPosition": Empty,
        "GetSweepData": SweepDataListMessage,
        "SetSweepData": Empty,
        "GetType": SimulationSetupTypeMessage,
    },
    "ansys.api.edb.v1.SIWaveDCIRSimulationSettingsService": {
        "GetIcepakTempFile": StringValue,
        "SetIcepakTempFile": Empty,
        "GetSourceTermsToGround": SourceTermsToGroundMessage,
        "SetSourceTermsToGround": Empty,
        "GetExportDCThermalData": BoolValue,
        "SetExportDCThermalData": Empty,
        "GetImportThermalData": BoolValue,
        "SetImportThermalData": Empty,
        "GetFullDCReportPath": StringValue,
        "SetFullDCReportPath": Empty,
        "GetViaReportPath": StringValue,
        "SetViaReportPath": Empty,
        "GetPerPinResPath": StringValue,
        "SetPerPinResPath": Empty,
        "GetDCReportConfigFile": StringValue,
        "SetDCReportConfigFile": Empty,
        "GetDCReportShowActiveDevices": BoolValue,
        "SetDCReportShowActiveDevices": Empty,
        "GetPerPinUsePinFormat": BoolValue,
        "SetPerPinUsePinFormat": Empty,
        "GetUseLoopResForPerPin": BoolValue,
        "SetUseLoopResForPerPin": Empty,
    },
    "ansys.api.edb.v1.SIWavePSIGeneralSettingsService": {
        "GetPISliderPos": SIWavePSISimulationPreferenceMessage,
        "SetPISliderPos": Empty,
        "GetSIWavePSIModelType": SIWavePSIModelTypeMessage,
        "SetSIWavePSIModelType": Empty,
        "GetMinPlaneAreaToMesh": StringValue,
        "SetMinPlaneAreaToMesh": Empty,
        "GetMinVoidAreaToMesh": StringValue,
        "SetMinVoidAreaToMesh": Empty,
        "GetSnapLengthThreshold": StringValue,
        "SetSnapLengthThreshold": Empty,
        "GetIncludeEnhancedBondWireModeling": BoolValue,
        "SetIncludeEnhancedBondWireModeling": Empty,
        "GetSurfaceRoughnessModel": SurfaceRoughnessModelMessage,
        "SetSurfaceRoughnessModel": Empty,
        "GetRMSSurfaceRoughness": StringValue,
        "SetRMSSurfaceRoughness": Empty,
        "GetPerformERC": BoolValue,
        "SetPerformERC": Empty,
    },
    "ansys.api.edb.v1.SIWavePSINetProcessingSettingsService": {
        "GetAutoSelectNetsForSimulation": BoolValue,
        "SetAutoSelectNetsForSimulation": Empty,
        "GetIgnoreDummyNetsForSelectedNets": BoolValue,
        "SetIgnoreDummyNetsForSelectedNets": Empty,
        "GetIncludeNets": StringsMessage,
        "SetIncludeNets": Empty,
    },
    "ansys.api.edb.v1.SIWavePSIPowerGroundNetsService": {
        "GetImprovedLossModel": ImprovedLossModelMessage,
        "SetImprovedLossModel": Empty,
        "GetAutoDetectIgnoreSmallHolesMinDiameter": BoolValue,
        "SetAutoDetectIgnoreSmallHolesMinDiameter": Empty,
        "GetIgnoreSmallHolesMinDiameter": StringValue,
        "SetIgnoreSmallHolesMinDiameter": Empty,
    },
    "ansys.api.edb.v1.SIWavePSISignalNetsSettingsService": {
        "GetSignalNetsErrorTolerance": ErrorToleranceMessage,
        "SetSignalNetsErrorTolerance": Empty,
        "GetSignalNetsConductorModeling": ConductorModelingMessage,
        "SetSignalNetsConductorModeling": Empty,
        "GetSignalNetsIncludeImprovedLossHandling": BoolValue,
        "SetSignalNetsIncludeImprovedLossHandling": Empty,
        "GetSignalNetsIncludeImprovedDielectricFillRefinement": BoolValue,
        "SetSignalNetsIncludeImprovedDielectricFillRefinement": Empty,
    },
    "ansys.api.edb.v1.SIWaveGeneralSettingsService": {
        "GetUseSISettings": BoolValue,
        "SetUseSISettings": Empty,
        "GetUseCustomSettings": BoolValue,
        "SetUseCustomSettings": Empty,
        "GetSISliderPos": UInt64Value,
        "SetSISliderPos": Empty,
        "GetPISliderPos": UInt64Value,
        "SetPISliderPos": Empty,
    },
    "ansys.api.edb.v1.SIWaveAdvancedSettingsService": {
        "GetIncludeCoPlaneCoupling": BoolValue,
        "SetIncludeCoPlaneCoupling": Empty,
        "GetIncludeInterPlaneCoupling": BoolValue,
        "SetIncludeInterPlaneCoupling": Empty,
        "GetIncludeSplitPlaneCoupling": BoolValue,
        "SetIncludeSplitPlaneCoupling": Empty,
        "GetIncludeFringePlaneCoupling": BoolValue,
        "SetIncludeFringePlaneCoupling": Empty,
        "GetIncludeTracePlaneCoupling": BoolValue,
        "SetIncludeTracePlaneCoupling": Empty,
        "GetCrossTalkThreshold": StringValue,
        "SetCrossTalkThreshold": Empty,
        "GetMaxCoupledLines": UInt64Value,
        "SetMaxCoupledLines": Empty,
        "GetMinVoidArea": StringValue,
        "SetMinVoidArea": Empty,
        "GetMinPadAreaToMesh": StringValue,
        "SetMinPadAreaToMesh": Empty,
        "GetMinPlaneAreaToMesh": StringValue,
        "SetMinPlaneAreaToMesh": Empty,
        "GetSnapLengthThreshold": StringValue,
        "SetSnapLengthThreshold": Empty,
        "GetMeshAutomatic": BoolValue,
        "SetMeshAutomatic": Empty,
        "GetMeshFrequency": StringValue,
        "SetMeshFrequency": Empty,
        "GetAcDcMergeMode": ACDCMergeModeMessage,
        "SetAcDcMergeMode": Empty,
        "Get3DReturnCurrentDistribution": BoolValue,
        "Set3DReturnCurrentDistribution": Empty,
        "GetIncludeVISources": BoolValue,
        "SetIncludeVISources": Empty,
        "GetIncludeInfGnd": BoolValue,
        "SetIncludeInfGnd": Empty,
        "GetInfGndLocation": StringValue,
        "SetInfGndLocation": Empty,
        "GetPerformERC": BoolValue,
        "SetPerformERC": Empty,
        "GetIgnoreNonFunctionalPads": BoolValue,
        "SetIgnoreNonFunctionalPads": Empty,
    },
    "ansys.api.edb.v1.SIWaveDCSettingsService": {
        "GetUseDCCustomSettings": BoolValue,
        "SetUseDCCustomSettings": Empty,
        "GetComputeInductance": BoolValue,
        "SetComputeInductance": Empty,
        "GetPlotJV": BoolValue,
        "SetPlotJV": Empty,
        "GetContactRadius": StringValue,
        "SetContactRadius": Empty,
        "GetDCSliderPos": UInt64Value,
        "SetDCSliderPos": Empty,
    },
    "ansys.api.edb.v1.SIWaveDCAdvancedSettingsService": {
        "GetDCMinPlaneAreaToMesh": StringValue,
        "SetDCMinPlaneAreaToMesh": Empty,
        "GetDCMinVoidAreaToMesh": StringValue,
        "SetDCMinVoidAreaToMesh": Empty,
        "GetMaxInitMeshEdgeLength": StringValue,
        "SetMaxInitMeshEdgeLength": Empty,
        "GetPerformAdaptiveRefinement": BoolValue,
        "SetPerformAdaptiveRefinement": Empty,
        "GetMaxNumPasses": UInt64Value,
        "SetMaxNumPasses": Empty,
        "GetMinNumPasses": UInt64Value,
        "SetMinNumPasses": Empty,
        "GetPercentLocalRefinement": UInt64Value,
        "SetPercentLocalRefinement": Empty,
        "GetEnergyError": DoubleValue,
        "SetEnergyError": Empty,
        "GetMeshBws": BoolValue,
        "SetMeshBws": Empty,
        "GetRefineBws": BoolValue,
        "SetRefineBws": Empty,
        "GetMeshVias": BoolValue,
        "SetMeshVias": Empty,
        "GetRefineVias": BoolValue,
        "SetRefineVias": Empty,
        "GetNumBwSides": UInt64Value,
        "SetNumBwSides": Empty,
        "GetNumViaSides": UInt64Value,
        "SetNumViaSides": Empty,
    },
    "ansys.api.edb.v1.SIWaveSParameterSettingsService": {
        "GetUseStateSpace": BoolValue,
        "SetUseStateSpace": Empty,
        "GetInterpolation": SParamInterpolationMessage,
        "SetInterpolation": Empty,
        "GetExtrapolation": SParamExtrapolationMessage,
        "SetExtrapolation": Empty,
        "GetDCBehavior": SParamDCBehaviorMessage,
        "SetDCBehavior": Empty,
    },
    "ansys.api.edb.v1.SolderBallPropertyService": {
        "Create": EDBObjMessage,
        "GetShape": PadstackDefDataSolderballShapeMessage,
        "SetShape": Empty,
        "GetPlacement": PadstackDefDataSolderballPlacementMessage,
        "SetPlacement": Empty,
        "GetDiameter": DiameterMessage,
        "SetDiameter": Empty,
        "GetHeight": ValueMessage,
        "SetHeight": Empty,
        "GetMaterialName": StringValue,
        "SetMaterialName": Empty,
        "UsesSolderball": BoolValue,
        "Clone": EDBObjMessage,
    },
    "ansys.api.edb.v1.SpiceModelService": {
        "Create": EDBObjMessage,
        "GetProperties": SpiceModelMessage,
        "SetModelPath": Empty,
        "SetModelName": Empty,
        "SetSubCkt": Empty,
        "AddTerminalPinPair": Empty,
        "RemoveTerminalPinPair": Empty,
        "GetTerminalPinPairs": StringMapMessage,
    },
    "ansys.api.edb.v1.StackupLayerService": {
        "Create": EDBObjMessage,
        "GetNegative": BoolValue,
        "SetNegative": Empty,
        "GetThickness": ValueMessage,
        "SetThickness": Empty,
        "GetLowerElevation": ValueMessage,
        "SetLowerElevation": Empty,
        "GetUpperElevation": ValueMessage,
        "GetMaterial": StringValue,
        "SetMaterial": Empty,
        "GetFillMaterial": StringValue,
        "SetFillMaterial": Empty,
        "SetRoughnessEnabled": Empty,
        "IsRoughnessEnabled": BoolValue,
        "GetRoughnessModel": RoughnessModelMessage,
        "SetRoughnessModel": Empty,
        "IsEtchFactorEnabled": BoolValue,
        "SetEtchFactorEnabled": Empty,
        "SetEtchFactor": Empty,
        "GetEtchFactor": ValueMessage,
        "SetEtchNetClass": Empty,
        "GetEtchNetClass": EtchNetClassMessage,
        "GetUseSolverProperties": BoolValue,
        "SetUseSolverProperties": Empty,
        "GetHFSSSolverProperties": HFSSSolverPropertiesMessage,
        "SetHFSSSolverProperties": Empty,
        "GetReferencingViaLayerIds": ViaLayerIdsMessage,
    },
    "ansys.api.edb.v1.Structure3DService": {
        "Create": EDBObjMessage,
        "GetMaterial": StringValue,
        "SetMaterial": Empty,
        "GetThickness": ValueMessage,
        "SetThickness": Empty,
        "GetMeshClosureProp": ClosureMessage,
        "SetMeshClosureProp": Empty,
    },
    "ansys.api.edb.v1.SParameterModelService": {
        "Create": EDBObjMessage,
        "GetProperties": SParameterModelMessage,
        "SetComponentModelName": Empty,
        "SetReferenceNet": Empty,
    },
    "ansys.api.edb.v1.TechnologyDefService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "Delete": Empty,
        "GetTechFile": StringValue,
        "GetGFDFile": StringValue,
        "GetLayerFile": StringValue,
        "GetName": StringValue,
        "GetIsCreateBackplane": BoolValue,
    },
    "ansys.api.edb.v1.TerminalService": {
        "FindByName": EDBObjMessage,
        "GetParams": TermParamsMessage,
        "SetParams": Empty,
        "GetProductSolvers": TermProductSolversMessage,
        "SetProductSolverOptions": Empty,
    },
    "ansys.api.edb.v1.TerminalInstanceService": {
        "Create": EDBObjMessage,
        "GetOwningCellInstance": EDBObjMessage,
        "GetDefinitionTerminal": EDBObjMessage,
        "GetDefinitionTerminalName": StringValue,
    },
    "ansys.api.edb.v1.TerminalInstanceTerminalService": {
        "Create": EDBObjMessage,
        "GetTerminalInstance": EDBObjMessage,
        "SetTerminalInstance": Empty,
    },
    "ansys.api.edb.v1.TextService": {
        "Create": EDBObjMessage,
        "GetTextData": TextDataMessage,
        "SetTextData": Empty,
    },
    "ansys.api.edb.v1.TransformService": {
        "Rotate": PointMessage,
        "Create": EDBObjMessage,
        "GetScale": ValueMessage,
        "SetScale": Empty,
        "GetMirror": BoolValue,
        "SetMirror": Empty,
        "GetRotation": ValueMessage,
        "SetRotation": Empty,
        "GetOffsetX": ValueMessage,
        "SetOffsetX": Empty,
        "GetOffsetY": ValueMessage,
        "SetOffsetY": Empty,
        "TransformPlus": EDBObjMessage,
        "IsIdentity": BoolValue,
        "TransformPoint": PointMessage,
        "TransformPolygon": PolygonDataMessage,
    },
    "ansys.api.edb.v1.Transform3DService": {
        "CreateIdentity": EDBObjMessage,
        "CreateCopy": EDBObjMessage,
        "CreateOffset": EDBObjMessage,
        "CreateCenterScale": EDBObjMessage,
        "CreateRotationFromAngle": EDBObjMessage,
        "CreateRotationFromAxis": EDBObjMessage,
        "CreateRotationFromAxisAndAngle": EDBObjMessage,
        "CreateRotationFromToAxis": EDBObjMessage,
        "CreateTransform2D": EDBObjMessage,
        "OperatorPlus": EDBObjMessage,
        "TransformPoint": CPos3DMessage,
        "GetZYXRotation": CPos3DMessage,
        "GetAxis": CPos3DTripleMessage,
        "Transpose": Empty,
        "Invert": Empty,
        "IsIdentity": BoolValue,
        "IsEqual": BoolValue,
        "GetScaling": CPos3DMessage,
        "GetShift": CPos3DMessage,
        "SetMatrix": Empty,
        "GetMatrix": DoublesMessage,
    },
    "ansys.api.edb.v1.ValueService": {
        "CreateValue": ValueMessage,
        "GetDouble": DoubleValue,
        "GetComplex": ComplexMessage,
    },
    "ansys.api.edb.v1.VariableServerService": {
        "AddVariable": Empty,
        "AddMenuVariable": Empty,
        "DeleteVariable": Empty,
        "SetVariableValue": Empty,
        "GetVariableValue": ValueMessage,
        "IsParameter": BoolValue,
        "GetAllVariableNames": GetVariableNamesMessage,
        "GetVariableDesc": StringValue,
        "SetVariableDesc": Empty,
    },
    "ansys.api.edb.v1.ViaGroupService": {
        "CreateWithPrimitives": EDBObjCollectionMessage,
        "CreateWithOutline": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "GetOutline": PolygonDataMessage,
        "GetConductorPercentage": DoubleValue,
        "IsPersistent": BoolValue,
    },
    "ansys.api.edb.v1.ViaLayerService": {
        "Create": EDBObjMessage,
        "GetRefLayerName": StringValue,
        "SetRefLayer": Empty,
        "GetIsTSV": BoolValue,
        "SetIsTSV": Empty,
        "AddOxideLayers": Empty,
        "RemoveOxideLayers": Empty,
        "GetNumOxideLayers": UInt64Value,
        "GetOxideLayers": OxideLayersMessage,
        "SetOxideLayerData": Empty,
    },
    "ansys.api.edb.v1.VoltageRegulatorService": {
        "Create": EDBObjMessage,
        "GetName": StringValue,
        "SetName": Empty,
        "IsActive": BoolValue,
        "SetIsActive": Empty,
        "GetVoltage": ValueMessage,
        "SetVoltage": Empty,
        "GetLoadRegulationCurrent": ValueMessage,
        "SetLoadRegulationCurrent": Empty,
        "GetLoadRegulationPercent": ValueMessage,
        "SetLoadRegulationPercent": Empty,
        "GetPosRemoteSensePin": EDBObjMessage,
        "SetPosRemoteSensePin": Empty,
        "GetNegRemoteSensePin": EDBObjMessage,
        "SetNegRemoteSensePin": Empty,
        "GetNPowerModules": Int64Value,
        "GetNActivePowerModules": Int64Value,
        "GetPowerModule": PowerModuleMessage,
        "GetAllPowerModules": PowerModulesMessage,
        "AddPowerModule": Empty,
        "RemovePowerModule": Empty,
        "AddPowerModules": Empty,
        "RemovePowerModules": Empty,
        "RemoveAllPowerModules": Empty,
    },
}


def get_rpc_response_type(service_name, rpc_name):
    """Get the response type for the provided service and RPC names."""
    service_map = rpc_response_map.get(service_name)
    return service_map.get(rpc_name) if service_map is not None else None
